#!/usr/bin/env python
# coding: utf-8

"""
Author : Gautam G
Created on 23 SEP 19
""";

import spacy
import nltk
from nltk.tokenize.toktok import ToktokTokenizer
import re
from bs4 import BeautifulSoup
from negationDictionay import negation_MAP
from correctionDictionary import correction_DIC
import unicodedata



nlp = spacy.load('en', parse = False, tag=False, entity=False)
tokenizer = ToktokTokenizer()
stopword_list = nltk.corpus.stopwords.words('english')
stopword_list.remove('no')
stopword_list.remove('not')



# Cleaning Text - strip HTML
def strip_html_tags(text):
    soup = BeautifulSoup(text, "html.parser")
    stripped_text = soup.get_text()
    return stripped_text


# Removing accented characters
def remove_accented_chars(text):
    text = unicodedata.normalize('NFKD', text).encode('ascii', 'ignore').decode('utf-8', 'ignore')
    return text


# Expanding Contractions
def expand_contractions(text, contraction_mapping=negation_MAP):
    
    contractions_pattern = re.compile('({})'.format('|'.join(contraction_mapping.keys())), 
                                      flags=re.IGNORECASE|re.DOTALL)
    def expand_match(contraction):
        match = contraction.group(0)
        first_char = match[0]
        expanded_contraction = contraction_mapping.get(match)                                    if contraction_mapping.get(match)                                     else contraction_mapping.get(match.lower())                       
        expanded_contraction = first_char+expanded_contraction[1:]
        return expanded_contraction
        
    expanded_text = contractions_pattern.sub(expand_match, text)
    expanded_text = re.sub("'", "", expanded_text)
    return expanded_text


# Removing Special Characters
def remove_special_characters(text):
    text = re.sub('[^a-zA-Z0-9\s]', '', text)
    return text


# Lemmatizing text
def lemmatize_text(text):
    text = nlp(text)
    text = ' '.join([word.lemma_ if word.lemma_ != '-PRON-' else word.text for word in text])
    return text


# Removing Stopwords
def remove_stopwords(text, is_lower_case=False):
    tokens = tokenizer.tokenize(text)
    tokens = [token.strip() for token in tokens]
    if is_lower_case:
        filtered_tokens = [token for token in tokens if token not in stopword_list]
    else:
        filtered_tokens = [token for token in tokens if token.lower() not in stopword_list]
    filtered_text = ' '.join(filtered_tokens)    
    return filtered_text


# Removing single charcters from the sentence
def remove_singleChar(text):
    text = text.split()
    text =  (s for s in text if len(s) > 1)
    return ' '.join(text)


# Removing numbers from sentence
def remove_number(text):
    pat4 = '[0-9]+'
    text = re.sub(pat4,'',text)
    return text


def removeAt(text):
    pat3 = '@[^ ]+'
    text = re.sub(pat3,'',text)
    return text

def removeHasTAg(text):
    pat = '#[^ ]+'
    text = re.sub(pat,'',text)
    return text

def removeHttp(text):
    pat = 'http[^ ]+'
    text = re.sub(pat,'',text)
    return text

def removeWww(text):
    pat = 'www.[^ ]+'
    text = re.sub(pat,'',text)
    return text

def removeEmail(text):
    pat = '\S+@\S+'
    text = re.sub(pat,'',text)
    return text

def wordCorrection(text):
    wordCorrection_pattern = re.compile(r'\b(' + '|'.join(correction_DIC.keys()) + r')\b')
    text = wordCorrection_pattern.sub(lambda x: correction_DIC[x.group()], text)
    return text

def normalize_corpus(corpus, html_stripping=True, contraction_expansion=True,
                     accented_char_removal=True, text_lower_case=True, 
                     text_lemmatization=True, special_char_removal=True, 
                     stopword_removal=True):
    
    normalized_corpus = []
    
    for doc in corpus:
#         print('doc',doc)
        if html_stripping:
            doc = strip_html_tags(doc)
#             print('html_stripping',doc)
        if accented_char_removal:
            doc = remove_accented_chars(doc)
#             print('accented_char_removal',doc)
            
        if contraction_expansion:
            doc = expand_contractions(doc)
#             print('contraction_expansion',doc)
            
        if text_lower_case:
            doc = doc.lower()
#             print('lower0',doc)
        # remove extra newlines
        doc = re.sub(r'[\r|\n|\r\n]+', ' ',doc)
#         print('lower1',doc)
        # insert spaces between special characters to isolate them    
        special_char_pattern = re.compile(r'([{.(-)!}])')
        doc = special_char_pattern.sub(" \\1 ", doc)
#         print('lower2',doc)
        if text_lemmatization:
            doc = lemmatize_text(doc)
#             print('lemmatize_text',doc)
            
        if special_char_removal:
            doc = remove_special_characters(doc)  
#             print('remove_special_characters',doc)
        # remove extra whitespace
        doc = re.sub(' +', ' ', doc)
#         print('remove_special_characters',doc)
        if stopword_removal:
            doc = remove_stopwords(doc, is_lower_case=text_lower_case)
            
        normalized_corpus.append(doc)
        
    return normalized_corpus


def text_preprocess(input_text, html_stripping=True, contraction_expansion=True,
                     accented_char_removal=True, text_lower_case=True, word_Correction=True,
                     text_lemmatization=True, special_char_removal=True, 
                     stopword_removal=True,removeSingleChar=True,removeNumber=True,
                    at_charWord=True,hastag=True,http=True,www=True,email=True):
    
    """input_text -> takes input string
       html_stripping=True -> if true removes html tag by default its True
       contraction_expansion=True -> if true converts negation words to normal by default its True
       accented_char_removal=True -> if true converts accented chars to normal by default its True
       text_lower_case=True -> if true converts text to lower case by default its True
       word_Correction -> this function checks spell of words and corrects
       text_lemmatization=True -> if true converts words to there root form by default its True
       special_char_removal=True -> if true remove special characters by default its True
       stopword_removal=True -> if true removes stopwords by default its True
       removeSingleChar=True -> if true removes single character default its True
       removeNumber=True -> if true removes numbers from text by default its True
       at_charWord=True -> if true remove words with '@' by default its True
       hastag=True -> if true remove words with '#' by default its True
       http=True -> if true remove http sites by default its True
       www=True -> if true remove wwww.com by default its True
       email=True -> if true remove gmails by default its True
                     """

    if html_stripping:
        doc = strip_html_tags(input_text)
        
    if accented_char_removal:
        doc = remove_accented_chars(doc)
            
    if contraction_expansion:
        doc = expand_contractions(doc)
            
    if text_lower_case:
        doc = doc.lower()

    if word_Correction:
        doc = wordCorrection(doc)

    if at_charWord:
        doc = removeAt(doc)

    if hastag:
        doc = removeHasTAg(doc)

    if http:
        doc = removeHttp(doc)

    if www:
        doc = removeWww(doc)

    if email:
        doc = removeEmail(doc)

#         # remove extra newlines
    doc = re.sub(r'[\r|\n|\r\n]+', ' ',doc)
#         # insert spaces between special characters to isolate them    
    special_char_pattern = re.compile(r'([{.(-)!}])')
    doc = special_char_pattern.sub(" \\1 ", doc)
    if text_lemmatization:
        doc = lemmatize_text(doc)
            
    if special_char_removal:
        doc = remove_special_characters(doc)  
#         # remove extra whitespace
    doc = re.sub(' +', ' ', doc)
    if stopword_removal:
        doc = remove_stopwords(doc, is_lower_case=text_lower_case)

    if removeNumber:
        doc = remove_number(doc)
    
    if remove_singleChar:
        doc =  remove_singleChar(doc)

    return doc



def version():
    return" Author : Gautam G Created on 23 SEP 19 updated on 15 NOV 19 ver3"
